; Contains Licensed Material Copyright (C) 1987 Ansa Software -- MJP

ApName=sdir()+"DemoApp"
Createlib ApName

; This procedure controls the entire data entry session.  Because a data entry
; procedure is typically a subsystem of an application, we can assume that all
; initialization is done at the beginning of the application by the main
; procedure.  The following commands were issued by such a procedure (during
; the execution phase):
;
;     Readlib "Toolkit" InitWait,SetKeycodes,SetPopup
;     InitWait("FldSpec","KeySpec")  ;Initialize the DoWait command
;     SetKeycodes()                  ;Initialize keycode variables such as TKUp
;     SetPopup("Carriers")           ;Prepare a popup menu from info in Carriers
;     Release Procs InitWait,SetKeycodes,SetPopup ;Init procs no longer needed
;     AreaCode="(415)"               ;Default if telephone number is 7 digits
;     TaxRate=.0725                  ;Current tax rate (7.25%)
;
; In order to invoke the data entry procedure, simply execute the following two
; commands (these are within the body of the main procedure):
;
;     Readlib "DemoApp" DataEntry
;     DataEntry()
;
; Upon return from the data entry procedure, the procedure may be released, and
; the application's main menu should be invoked.
;
Proc DataEntry()
   Readlib "DemoApp" Reqd,FillDate,DateLeave,CustLeave,ResetSpecs,ChangeDisc,
                     SpecialKey,FindPart,ChangeQuan,DoTotal,FillSubtotal,BadID,
                     BadCarrier,GetCust,NoUp,NoUpLeft,NoDownRight,NormalPhone,
                     FillPhone
   Readlib TKLibName EditMenu,LookupSelect,RecurseWait
   Setswap 16000
   Empty "TKEntry"
   View "Cust"       ;Cust is placed on the workspace to facilitate interactive
                     ; updating of the file
   CoEdit "TKEntry"  ;CoEdit is used because it will update the primary key for
                     ; the Cust table the instant a new record is entered.  (Not
                     ; at the end of the data entry session)
   Pickform "F"
   Setswap 12000
   Prompt "Entering invoice information.",
          "Press [F1] for help, [F2] when done, [F10] for menu."
   DoWait("")
   Clear
   Prompt
   Release Procs Reqd,FillDate,DateLeave,CustLeave,BadCarrier,ChangeDisc,
                 SpecialKey,FindPart,ChangeQuan,DoTotal,FillSubtotal,BadID,NoUp,
                 GetCust,NoUpLeft,NoDownRight,NormalPhone,FillPhone,ResetSpecs,
                 LookupSelect,EditMenu,RecurseWait
   ;This is where the procedure to post the records from TKEntry should go.
   ; Notice that this is after all of the other procedures are released, as the
   ; queries used to post the information may require more memory than would
   ; otherwise be available.
   ;
   ;Another way to obtain more memory would be to allow Paradox to automatically
   ; swap out procedures as needed. This would be accomplished by using SETSWAP
   ; to define the amount of memory needed by the posting procedure, and then
   ; reading in that procedure. (Paradox would then swap out as many procs as
   ; necessary to maintain the required memory.)
   ;
   ;The posting process should begin with a "NewEntries" multiadd to post
   ; the one-to-one relationships, after which the KeyViol table should be
   ; subtracted from the TKEntry table (if a KeyViol table exists). Next, the
   ; one-to-many information should be posted using a script similar to the
   ; sample "Post" script. Last, the KeyViol table should be processed (if
   ; it exists). The manner in which the KeyViol table is dealt with is
   ; completely application dependent, and is thus beyond the scope of this
   ; example.
   Reset  ;Demo application doesn't post information- Clear workspace
Endproc
Writelib ApName DataEntry
Release Procs DataEntry

; This procedure will mimic the REQUIRED validity check, except that it will
; be less restrictive.  Unlike the validity check, this procedure will allow
; the menu to be displayed, help to be pressed, and edit to be cancelled
; regardless of the contents (or lack thereof) of the field.
;
; Also notice that if the field is left blank, a user-defined message is
; displayed rather than the Paradox default message.
;
; This procedure is called upon attempted departure from the Invoice # field.
;
Proc Reqd()
   If isblank([]) and TKChar<>TKDel and TKChar<>TKUndo and TKChar<>TKCancel
      Then TKAccept=False     ;Ignore movement key that was pressed (Stay here)
           TKMessage="An invoice number must be provided."
   Endif
Endproc
Writelib ApName Reqd
Release Procs Reqd

; This procedure will automatically put today's date into the Date field if
; that field is blank upon entry.  Further, once it has filled in the date,
; the procedure will leave the field, moving in the same direction from which
; it arrived.
;
; Although the procedure itself doesn't contain the TODAY() function, it is
; able to fill in the date because Today has been defined as a default validity
; check for the field.  Thus the act of moving out of the field will cause the
; date to be filled in.  This is a good example of how the form itself can
; interact with these procedures in order to provide extreme power with minimal
; programming effort.
;
; This procedure is called upon arrival into the Date field.
;
Proc FillDate()
   If isblank([])
      Then If TKChar=TKReverseTab or TKChar=TKLeft ;Did we arrive from right?
              Then Left                            ; Yes- Continue along
              Else Right                           ; No-  Move to the right
           Endif          ;Remember, date will automatically be set by ValCheck
           ArriveField()  ;We have left the field explicitly (unexpectedly),
   Endif                  ; thus we must tell DoWait that we are arriving in
Endproc                   ; a new field.
Writelib ApName FillDate
Release Procs FillDate

; This procedure looks at the effect of each keystroke in order to determine
; whether the date has been completely filled in.  If the date is complete,
; then the procedure will cause the field to be left automatically.
;
; Please note that this is not a procedure which simply departs after a certain
; number of keystrokes (which would require dates to be entered in a format
; such as 07/01/87 rather than 7/1/87), but rather a procedure which is able
; to recognize a completed pattern.
;
; This procedure is called upon each "Regular" keystroke while in the Date
; field.
;
Proc DateLeave()
   Keypress TKChar       ;Accept the character- Paradox will deal with invalids
   If match(fieldstr(),"../../@@")  ;Is the field complete?
        or match(fieldstr(),"..-..-@@")
      Then TKChar=TKRight   ; Yes- Reset character.  Since TKAccept has not been
                            ;      set to False, DoWait will now act as if the
                            ;      user had actually pressed Right
      Else TKAccept=False   ; No-  Since we already keypressed the character, we
   Endif                    ;      want DoWait to ignore it
Endproc
Writelib ApName DateLeave
Release Procs DateLeave

; This procedure is responsible for maintaining the integrity of the summary
; information displayed at the bottom of the form as the customer number is
; entered or updated.  This is necessary because the default discount percent
; associated with each customer effects the bottom line figures.  Since the
; TableLookup validity check defined for this field may change the discount
; percent upon departure from the field, the summary information must be
; updated as well.
;
; This procedure is called upon attempted departure from the Customer ID field
; if the data in the field is valid.
;
Proc CustLeave()
   If not TKChanged    ;Do nothing if the customer number was not changed
          or TKChar=TKCancel
      Then Return
   Endif
   Echo Off
   Synccursor
   OldDisc=[Discount]   ;Remember discount so we can tell if it changes
   If isblank([])       ;If the customer ID is blank, we want all of the lookup
      Then []="\255"    ; fields to be blanked as well. "\255" is a dummy record
           Backspace    ; in Cust with blank lookup fields. The assignment will
                        ; blank the appropriate fields of TKEntry
   Endif
   ;NOTE: The Discount field in TKEntry has been defined as one of the fields
   ;      that will be filled in when the Customer ID field is entered. This
   ;      fillin has not taken place yet, however. To cause this to occur, we
   ;      must move out of the Customer ID field or assign a value to it. If we
   ;      don't do either before checking the discount, then the discount will
   ;      not have changed.
   []=[]                          ;Force fill-in to occur
   If OldDisc<>[Discount]         ;Has the discount percent changed?
      Then DoTotal()              ; Yes- Recalculate all summary information
   Endif
   If TKChar=TKEnter or TKChar=TKRight or TKChar=TKTab ;Moving to Discount?
      Then Moveto [Quan1]   ;Skip over the Discount field (It will be filled in
           ArriveField()    ;  (It will be filled in automatically)
   Endif                    ;Otherwise simply allow key to be passed on through
Endproc
Writelib ApName CustLeave
Release Procs CustLeave

; This procedure can perform several functions.  First, it puts up an
; application-sensitive message about invalid customer data rather than allowing
; Paradox to put up its default message.  Next, it provides a menu which allows
; the user to get help or to interactively add information to the customer
; master file.
;
; NOTE:  Allowing adhoc updates to the customer master file, although
;        appropriate in some applications, is generally not considered to be
;        sound practice as data entry errors and duplicate entries are bound
;        to occur in the master file.  This procedure is provided only as an
;        example of certain concepts such as how to call DoWait recursively.
;
; This procedure is called upon attempted departure from the Customer ID field
; when the data contained in that field does not pass all validity checks.
;
Proc BadID()
   If TKChar=TKDel or TKChar=TKUndo   ;We don't care about invalid data if one
                   or TKChar=TKCancel ; of these keys is pressed, as the value
      Then Return                     ; of the field becomes inconsequential
   Endif
   Echo Off
   Message "Customer number "+[]+" does not exist in master file."
   Showmenu
      "Reenter":"Reenter customer number.",
      "Add":"New customer-- Add to customer master file.",
      "Help":"Present a list of valid customers"
   To Choice
   Switch
      Case Choice="Add":
         GetCust()        ;Initiate data entry for customer master file
      Case Choice="Help":
         TKChar=TKHelp    ;Tell DoWait the key was Help rather than what it was
      Otherwise:
         TKAccept=False   ;Reenter or Esc- Don't accept the key (Stay in field)
   Endswitch
Endproc
Writelib ApName BadID
Release Procs BadID

; This procedure invokes data entry within the customer master file.  It will
; allow one and only one record to be entered, and severely limits what the
; user is allowed to do.  (No menu, for example.)  These limitations are
; intended to make improper data entry into the master file very unlikely.
;
; The procedure is called by BadID if the user wishes to add a currently
; invalid customer ID to the master file.
;
Proc GetCust()
   CustID=[]       ;Remember the ID number for later
   []=""           ;Blank the ID, as we can't move out of an image if the
                   ; current field does not pass all validity checks
   UpImage         ; Move up into the customer table
   End             ; Open up a new record, positioned in the first field
   Down
   []=CustID       ;Place the potential new ID into the customer table
   Right
   PickForm "F"
   Prompt "Enter the customer information.",
          "Press [F2] to add the customer, or [Esc] to cancel."
   ;The DoWait procedure that follows will be called recursively (from within
   ; itself). When called in this manner, the procedure will automatically save
   ; all of the variables that DoWait passes to procedures, and restore them
   ; upon completion of the call. Thus variables such as TKChar, TKAccept, etc.
   ; will all maintain their current values
   RecurseWait("","ResetSpecs") ;Invoke data entry.
   If Retval=TKEsc              ;Did user cancel?
      Then Del              ; Yes- Remove unwanted record from the Cust table
           TKAccept=False   ;      Do not accept departure char (Stay in field)
      Else LockKey          ; No-  Unlock the record (which will post it)
   Endif
   Prompt "Entering invoice information.",
          "Press [F1] for help, [F10] for menu."
   DownImage       ;Move back to the TKEntry table
   FormKey
   Typein CustID   ;Put the ID back into current field. (It won't be valid if
                   ; user escaped from customer entry, so we can't assign)
   If TKAccept     ;A new customer name has been entered
      Then TKChanged=True  ;Name must be valid- Process it
           CustLeave()
   Endif
Endproc
Writelib ApName GetCust
Release Procs GetCust

; This procedure resets the field and key specifications.  Actually, since
; these variables are declared as private to GetCust, the procedure is really
; setting the variables rather than resetting them.
;
; The variables defined here were not set by hand, but rather by the field and
; key definition programs in the toolkit.  After the field spec and the key
; spec had been created, both files were read in and combined to form this
; procedure (which will execute much more rapidly than playing the two non-
; proceduralized scripts).
;
; This procedure is called by GetCust each time we enter data into the customer
; master file.
;
Proc ResetSpecs()
   TKNegKey="EIIIIIIIIIIIIIIMIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIREIII"+
            "IIIIIIIIMIIMIMIIMIIIIIIIIIIIIIIIIIIIIIIIIIIIRIIIIIMMMMIMIIIIIIII"+
            "IIIII"
   TKPosKey="RRRRRRRRMRRRMRRRRRRRIRRRRRERRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR"+
            "RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR"+
            "RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR"+
            "RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR"
   Array TKAction[10]
   TKAction[1] = ""  TKAction[2] = ""  TKAction[3] = "D"  TKAction[4] = "D"
   TKAction[5] = ""  TKAction[6] = ""  TKAction[7] = ""   TKAction[8] = ""
   TKAction[9] = "DK"  TKAction[10] = "D"
   Array TKGoodDepart[10]
   TKGoodDepart[3] = "NoUpLeft"  TKGoodDepart[4] = "NoUp"
   TKGoodDepart[9] = "NormalPhone"  TKGoodDepart[10] = "NoDownRight"
   Array TKKeystroke[9]
   TKKeystroke[9] = "FillPhone"
Endproc
Writelib ApName ResetSpecs
Release Procs ResetSpecs

; This procedure does not allow the up arrow to be pressed.  It is used to
; disallow departure from a record in form view, and is called upon departure
; from the First Name field of the Customer table.
;
Proc NoUp()
   If TKChar=TKUp
      Then Beep
           TKAccept=False
   Endif
Endproc
Writelib ApName NoUp
Release Procs NoUp

; This procedure does not allow the up or left arrows to be pressed.  It is
; used to disallow departure from a record in form view, and is called upon
; departure from the Last Name field of the Customer table.
;
Proc NoUpLeft()
   If TKChar=TKUp or TKChar=TKLeft or TKChar=TKReverseTab
      Then Beep
           TKAccept=False
   Endif
Endproc
Writelib ApName NoUpLeft
Release Procs NoUpLeft

; This procedure does not allow the down or right arrows to be pressed.  It is
; used to disallow departure from a record in form view, and is called upon
; departure from the Discount field of the Customer table.
;
Proc NoDownRight()
   If TKChar=TKEnter or TKChar=TKRight or TKChar=TKTab or TKChar=TKDown
      Then Beep
           TKAccept=False
   Endif
Endproc
Writelib ApName NoDownRight
Release Procs NoDownRight

; This procedure serves two purposes.  First, it enhances the PICTURE validity
; check by allowing after-the-fact reformatting of data in the field.  Second,
; it causes automatic departure from the field if the data has been filled in
; completely.
;
; The procedure is called every time a "Regular" key is pressed within the
; Telephone field of the Customer table.
;
Proc FillPhone()
   Switch
      Case TKChar<48 or TKChar>57:    ;Make sure the pending key is a digit
      Case len([])=8 and substr([],1,1)<>"("    ;If there is a digit pending
           and substr([],8,1)<>"-":             ; and the partial data looks
         []=substr([],1,7)+"-"+substr([],8,1)   ; like ###-####, then we want
                                                ; to change it to ###-###-#
      Case len([])=12 or len([])=11 and substr([],4,1)="-": ;Is field complete?
         Keypress TKChar  ;Accept the character which will complete the field
         TKChar=TKRight   ;Cause DoWait to act like the user pressed Right
   Endswitch
Endproc
Writelib ApName FillPhone
Release Procs FillPhone

; This procedure puts telephone numbers into a standard form.  The user has the
; option of entering telephone numbers in one of three formats:  (###)###-####,
; ###-###-####, or ###-####.  Yet we do not want to allow "randomly" formatted
; data to be placed in our tables.  Thus this procedure will cause all telephone
; numbers to be formatted as (###)###-####.
;
; Notice that the checks that are made here seem incomplete.  That is, the fact
; that the length of the string is 8 characters does not by itself mean that it
; is in the form ###-####.  (It could be (###)###, for example.)  The reason
; why the check is not more specific is that there is a picture defined for the
; field of (###)###-####,###-###{#,-####}.  Since this procedure is only called
; when the data in the field is completely valid, if this procedure is called
; and the length of the data is 8 characters, then it must look like ###-####.
;
; The procedure is called upon attempted departure from the Telephone field of
; the Customer table.
;
Proc NormalPhone()
   Switch
      Case len([])=12:                              ;###-###-####
         []="("+substr([],1,3)+")"+substr([],5,8)
      Case len([])=8:                               ;###-####
         []=AreaCode+[]  ;AreaCode is defined to be "(###)"
   Endswitch
Endproc
Writelib ApName NormalPhone
Release Procs NormalPhone

; This procedure updates summary information when the user modifies the Discount
; field.  It is called upon attempted departure from that field.
;
Proc ChangeDisc()
   If TKChanged                ;Has the data in the field been modified?
      Then Echo Off
           Synccursor
           DoTotal()
   Endif
Endproc
Writelib ApName ChangeDisc
Release Procs ChangeDisc

; This procedure puts up an application-dependent message rather than a Paradox
; message when the user attempts to leave the Ship Via field and the information
; in that field is not one of the valid choices.
;
Proc BadCarrier()
   TKMessage="Not a valid carrier.  Press [F1] for a list of options."
   ;Notice that we have not set TKAccept to False. Accepting a "Move" character
   ; when a field is invalid will simply cause Paradox to put up a default
   ; message. Since we have provided our own message, DoWait will act as if
   ; TKAccept was set to False and suppress Paradox's message
Endproc
Writelib ApName BadCarrier
Release Procs BadCarrier

; This procedure retrieves information pertinent to a given part number.  The
; information is pulled in from the Stock table, and consists of description
; and unit price.  Unit price is then multiplied by quantity (if not blank) to
; get extended price, which is then added to the subtotal.  Subtotal information
; is then used to update all summary figures.
;
; This procedure is called upon attempted departure from any of the Part fields.
;
Proc FindPart()
   If TKChanged      ;Do nothing unless the part number has changed
      Then Echo Off
           Synccursor
           Fldno=substr(field(),5,1)  ;Which Part field are we in (1-8)?
           Execute "Quan=[Quan"+Fldno+"] OldExt=[Extd"+Fldno+"]"
             ;Quan=[Quan#]
             ;OldExt=[Extd#]
           If isblank([])  ;Part number has been removed?
              Then Execute "[Desc"+Fldno+"]=\"\" [Price"+Fldno+
                           "]=blanknum() [Extd"+Fldno+"]=blanknum()"
                     ;[Desc#]=""
                     ;[Price#]=blanknum()
                     ;[Extd#]=blanknum()
                   If not isblank(OldExt)  ;If there used to be an extd price
                      Then [Subtotal]=[Subtotal]-OldExt ; then remove it
                           DoTotal()                    ; from summary info
                   Endif
              Else Part=[]  ;Part number exists, but has been changed
                   Help                       ;Bring up lookup table
                   Locate Part                ;Find part in lookup table
                   Desc=[Item]                ;Record Item Description
                   Price=[Price]              ;       Unit Price
                   Esc                        ;Remove lookup table
                   Execute "[Desc"+Fldno+"]=Desc\n[Price"+Fldno+"]=Price"
                     ;[Desc#]=Desc
                     ;[Price#]=Price
                   If not isblank(Quan)
                      Then FillSubtotal()     ;Update Extd Price and
                   Endif                      ; summary fields
           Endif
   Endif
Endproc
Writelib ApName FindPart
Release Procs FindPart

; This procedure updates extended price and summary fields when the quantity
; of a particular line item is changed.  It is called upon attempted departure
; from any of the quantity fields.
;
Proc ChangeQuan()
   If TKChanged    ;Do nothing unless the quantity has changed
      Then Fldno=substr(field(),5,1)  ;Which Quan field are we in (1-8)?
           Execute "Price=[Price"+Fldno+"] OldExt=[Extd"+Fldno+"]"
             ;Price=[Price#]
             ;OldExt=[Extd#]
           If isblank([])   ;Has quantity been removed?
              Then Echo Off
                   Synccursor
                   Execute "[Extd"+Fldno+"]=blanknum()" ;[Extd#]=blanknum()
                   If not isblank(OldExt)
                      Then [Subtotal]=[Subtotal]-OldExt  ;Update summary info
                           DoTotal()                     ; if necessary
                   Endif
              Else If not isblank(Price)
                      Then Echo Off
                           Synccursor
                           Quan=[]
                           FillSubtotal()  ;Update extd price and summary fields
                   Endif
           Endif
   Endif
Endproc
Writelib ApName ChangeQuan
Release Procs ChangeQuan

; This procedure is called whenever a "Special" key is pressed.  Because there
; can be many special keys defined in a given application, the first task of
; the procedure is to determine which key was pressed.  Once this has been
; determined, the special purpose processing may be accomplished easily.
;
; Two keys have been defined as being "Special" in this application.  They are
; the Help key and the Menu key.  This procedure is invoked whenever either of
; these is pressed.
;
Proc SpecialKey()
   Switch
      Case TKChar=TKHelp:                  ;The special key was Help
         Switch
            Case field()="Customer ID":          ;Help was pressed from ID field
               OldDisc=[discount]
               Help
               LookupSelect()                    ;Allow user to choose from Cust
               If Retval=TKDo_It!
                  Then Do_It!                    ;User selected an ID
                       Moveto [Quan1]            ;Skip Discount (it was filled)
                       If OldDisc<>[Discount]
                          Then DoTotal()         ;Update totals if appropriate
                       Endif
                       ArriveField()             ;We have moved to a new field
                  Else Esc
                       TKAccept=False            ;Don't accept the help key
               Endif
            Case match(field(),"Part@",Fldno):   ;Help for one of the Part flds
               Help
               LookupSelect()                    ;Accept choice from Stock
               If Retval=TKDo_It!
                  Then Part=[Part Number]        ;User selected a part number
                       Desc=[Item]               ;Record info from Stock table
                       Price=[Price]
                       Esc                       ;Exit from lookup help
                       Execute "Quan=[Quan"+Fldno+"] OldExt=[Extd"+Fldno+
                               "] [Part"+Fldno+"]=Part\n[Desc"+Fldno+
                               "]=Desc\n[Price"+Fldno+"]=Price"
                         ;Quan=[Quan#]
                         ;OldExt=[Extd#]
                         ;[Part#]=Part
                         ;[Desc#]=Desc
                         ;[Price#]=Price
                       If not isblank(Quan)
                          Then FillSubtotal() ;Update info if appropriate
                       Endif
                       Right            ;Move into next field
                       ArriveField()    ;Tell DoWait that we have arrived
                  Else Esc
                       TKAccept=False   ;Don't accept the Help key
               Endif
            Case field()="Ship Via":    ;Help for Ship Via field
               Popup(2,37,1,4)          ;Place a popup menu at screen
                                        ; coordinates (2,37), and display 4
               If isblank(Retval)       ; items at a time
                  Then TKAccept=False   ;No selection made- do nothing
                  Else []=Retval        ;Fill the field with the value (one
                       TKChar=TKRight   ; was chosen) and make DoWait act
               Endif                    ; as if Right was hit, not Help
         Endswitch
      Case TKChar=TKMenu:               ;The special key was Menu
         EditMenu()                     ;Create a Paradox-style menu
   Endswitch
Endproc
Writelib ApName SpecialKey
Release Procs SpecialKey

; This procedure updates tax and total information.  It should be called every
; time the subtotal field changes.
;
Proc DoTotal()
   If isblank([Subtotal])
      Then [Subtotal]=0   ;This avoids the problem of trying to add blank fields
   Endif
   If isblank([Discount])  ;If no discount is specified, assume it is zero
      Then [Discount $]=0
      Else ;The following code is functionally equivalent to:
           ;    [Discount $]=int([Subtotal]*[Discount])/100
           ; An explanation of why the "int" function was not used here is
           ; beyond the scope of this demo application
           If match([Subtotal]*[Discount]/100,"...@@..",A,B,C)
              Then [Discount $]=numval(A+"."+B+C)
              Else [Discount $]=[Subtotal]*[Discount]/100
           Endif
   Endif
   ;[Tax]=int([Subtotal]-[Discount $])*TaxRate*100)/100
   If match(([Subtotal]-[Discount $])*TaxRate,"...@@..",A,B,C)
      Then [Tax]=numval(A+"."+B+C)
      Else [Tax]=([Subtotal]-[Discount $])*TaxRate
   Endif
   [Total]=[Subtotal]-[Discount $]+[Tax]+60 ;The 60 is a fixed rate for freight
Endproc
Writelib ApName DoTotal
Release Procs DoTotal

; This procedure fills the current extended price field with the proper value
; (as defined by the variables Price and Quan), and then updates the subtotal
; information by adding this new value and subtracting the old value (stored
; in the variable OldExt).
;
; The procedure should be called every time a quantity or a part is entered
; (or changed).
;
Proc FillSubtotal()
   If isblank([Subtotal])
      Then [Subtotal]=0
   Endif
   NewExt=Price*Quan
   Execute "[Extd"+Fldno+"]=NewExt"
   If isblank(OldExt)
      Then [Subtotal]=[Subtotal]+NewExt
      Else [Subtotal]=[Subtotal]+NewExt-OldExt
   Endif
   DoTotal()                        ;Update tax and total info.
Endproc
Writelib ApName FillSubtotal
Release Procs FillSubtotal
