; SCRIPT:     PORTFOL.SC
; Version:    2.0
; Date:       2 1987
;
;
; This script defines the procedures that display a summary of holdings
; in a specific portfolio (as requested by the user). It functions
; in a similar manner to HOLDINGS.

PROC SpecificPort()
  PRIVATE x
  IF (NOT AskPortName())            ; Get a portfolio name from user
    THEN RETURN                     ; User didn't supply name, so return
  ENDIF
  WHILE (NOT CHARWAITING())         ; For as long as user doesn't press key
    CLEARALL                        ; | - clear the workspace
    SpecQuery1()                    ; | - put the portfolio query on workspace
    WHILE (TRUE)                    ; |
      LOCK "Price" WL               ; | - can we get the table?
      IF (retval) THEN QUITLOOP ENDIF ; yes, so go on
      SLEEP 500                     ; | no, so pause before retrying
    ENDWHILE                        ; |
    DO_IT!                          ; | - process the query
    UNLOCK "Price" WL               ; | - give up lock on table
    CLEARALL                        ; |
    VIEW "Answer"                   ; | - take the resultant ANSWER table and...
    MOVETO [Price * Shares]         ; |
    MOVETO [Ticker]                 ; |
    ECHO NORMAL                     ; | - turn on ECHO to see results
    PROMPT "Portfolio for " + portname + " as of " + Time(),
           "Total Portfolio value is $" +
           FORMAT("W12,AL,EC",csum("Answer", "Price * Shares")) +
           "   --      Press Any Key to Quit"
    ECHO OFF                        ; | - turn off ECHO so user doesn't see next query
 ENDWHILE                           ; v

; If we drop out of above loop, user must have pressed a key

 x = GETCHAR()                      ; Gobble up the keystroke
 CLEARALL                           ; And erase the workspace
 CLEAR                              ;    ...and canvas
ENDPROC

; SpecQuery1 puts the query that does all the work onto the workspace.
; Note:
;    - the tilde variable ~portname supplies the portfolio name
;    - calc _p * _sh calculates total value for a stock

PROC SpecQuery1()

QUERY

 Price |   TICKER    |  PRICE   |  LAST  |
       | Check _tick | Check _p | Check  |
       |             |          |        |
       |             |          |        |

 Holding | PORTFOLIO | SYMBOL |          SHARES          |
         | ~portname | _tick  | Check _sh, calc _p * _sh |
         |           |        |                          |
         |           |        |                          |

ENDQUERY
ENDPROC

; AskPortName does the job of requesting and accepting from the user the
; name of the portfolio to display a summary of.

PROC AskPortName()
  PRIVATE n,Nams,Prompts
  ; First we must get the names of the Portfolios
  MENU {Ask} SELECT "Holding"     ; HOLDING table has names of portfolios
  MOVETO [Portfolio]              ; Portfolio names are in Portfolio Field
  CHECK                           ; So CHECK that field
  DO_IT!                          ; and perform a query to get names into table
  n = NRECORDS("Answer")          ; User has n portfolios (names now in Answer)
  ARRAY Nams[n]                   ; Create an array to put the names into
  ARRAY Prompts[n]                ; And another array to store prompts for names
  SCAN                            ; SCAN through each record of Answer to fill arrays
    Nams[ RECNO() ]    = [Portfolio]
    Prompts[ RECNO() ] = "Use the " + [Portfolio] + " portfolio"
  ENDSCAN
  CLEARALL                              ; Clear the workspace
  SHOWARRAY Nams Prompts TO Portname    ; Display portfolio names in a Paradox-style menu
  RETURN (Portname <> "Esc")            ; Tell the world if we got a valid name
ENDPROC
